SystemOrganization addCategory: #FuelPackageLoader!
SystemOrganization addCategory: #'FuelPackageLoader-Metacello'!
SystemOrganization addCategory: #'FuelPackageLoader-Mappers'!

!Association methodsFor: '*FuelPackageLoader' stamp: 'MarianoMartinezPeck 9/23/2011 13:15'!
rebindOn: aCompiledMethod at: index
	"If the CompiledMethod was saved with a reference to a class that does exist, then its literal will be in the way of: #ClassNameThatDoesNotExist -> nil.
	So....if we try to rebind that #bindingOf: will answer nil. Hence, we will create an incorrect CompiledMethod where we have a nil where we should have a Association in the way of  #ClassNameThatDoesNotExist -> nil. "
	value isNil ifFalse: [
		aCompiledMethod 
			literalAt: index 
			put: (aCompiledMethod methodClass bindingOf: key)
	]
	! !

!Class methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 21:16'!
cleanUpClassVariables

	self classPool keys do: [ :key | classPool at: key put: nil ]! !

!Class methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 21:41'!
cleanUpInstanceVariables

	Class instSize + 1 to: self class instSize do:
		[:offset | self instVarAt: offset put: nil].
! !

!Class methodsFor: '*FuelPackageLoader' stamp: 'MarianoMartinezPeck 9/23/2011 16:44'!
copyWithLocalProtocol

	| copy |
	copy := self copy.

	copy organization: organization deepCopy.
	copy environment: environment.
	copy basicCategory: category.	
		
	copy traitComposition: self traitComposition.
	copy classSide traitComposition: self classSide traitComposition.
	
	copy removeForeignProtocol.
	copy classSide removeForeignProtocol.
	
	copy cleanUpClassVariables.
	copy cleanUpInstanceVariables.
	
	^copy! !

!Class methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 02:53'!
fuelSubstitution

	^self copyWithLocalProtocol ! !

FLDefaultMapper subclass: #FLPackageLoaderDefaultMapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPackageLoader-Mappers'!

!FLPackageLoaderDefaultMapper commentStamp: 'MartinDias 8/24/2011 00:08' prior: 0!
I am like the default mapper but with a special behavior with classes. I store a copy each class with just the local protocol.!

!FLPackageLoaderDefaultMapper methodsFor: 'visiting' stamp: 'MarianoMartinezPeck 9/23/2011 16:44'!
initialize
	super initialize.
	compiledMethodCluster := FLCompiledMethodCluster withNoSource.
	namedCompiledMethodCluster := FLNamedPrimitiveMethodCluster withNoSource.! !

!FLPackageLoaderDefaultMapper methodsFor: 'visiting' stamp: 'MartinDias 8/24/2011 00:18'!
visitClass: aClass  

	^(Smalltalk globals includes: aClass)
		ifTrue: [ super visitSubstitution: aClass ]
		ifFalse: [ super visitClass: aClass ]! !

!CompiledMethod methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 9/22/2011 02:10'!
rebindLiteralAt: index

	(self literalAt: index) rebindOn: self at: index! !

!CompiledMethod methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 9/22/2011 02:07'!
rebindLiterals 

	1 to: self numLiterals-2
		do: [ :index | self rebindLiteralAt: index ]! !

FLAnalyzer subclass: #FLPackageAnalyzer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPackageLoader'!

!FLPackageAnalyzer methodsFor: 'defaults' stamp: 'MartinDias 9/13/2011 21:59'!
defaultMapper

	^FLPackageLoaderDefaultMapper new! !

!TClassAndTraitDescription methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 00:27'!
removeCategorySilently: aCategory

	SystemChangeNotifier uniqueInstance doSilently: [ self removeCategory: aCategory ]! !

!ClassOrganizer methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 01:32'!
removeCategorySilently: cat 

	super removeCategory: cat! !

!ClassOrganizer methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 01:49'!
removeElementSilently: element

	^super removeElement: element! !

!MetacelloGoferLoad methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/23/2011 18:09'!
execute

	super execute.
	FLMetacelloStore default packagesHasBeenLoaded: (self model versions collect: [:version | version package name]).! !

!Behavior methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 9/22/2011 15:35'!
rebindMethodDictionary

	self methodDict valuesDo: [ :aCompiledMethod | 
		aCompiledMethod 
			methodClass: self;
			rebindLiterals. ]! !

Object subclass: #FLMetacelloStore
	instanceVariableNames: 'prefix capturing capturedPackageNames stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPackageLoader-Metacello'!

FLMetacelloStore class
	instanceVariableNames: 'default'!

!FLMetacelloStore commentStamp: 'MartinDias 8/23/2011 19:06' prior: 0!
I capture and export in Fuel files a Metacello load.

Example: loading Seaside 3

1) Install Fuel in a CLEAN Pharo 1.2.1 image:
Gofer new squeaksource: 'Fuel'; 
	package: 'Fuel';
	package: 'FuelTests';
	package: 'FuelPackageLoader';
	load.

2) Load the configuration you want to capture in this way:
FLMetacelloStore default 
	capturePackagesDuring: [
		Gofer new squeaksource: 'MetacelloRepository'; package: 'ConfigurationOfSeaside30'; load.
		(Smalltalk at: #ConfigurationOfSeaside30) project latestVersion load: #(Core)] 
	prefixed: 'seaside'.

= Load =

1) Install Fuel:
Gofer new squeaksource: 'Fuel'; 
	package: 'Fuel';
	package: 'FuelTests';
	package: 'FuelPackageLoader';
	load.

2) "Now run the loading script generated: copy and paste in a workspace the content of 'seaside-load-script.txt'."

3) "Start the server"
WAComancheAdaptor startOn: 8088.!

FLMetacelloStore class
	instanceVariableNames: 'default'!

!FLMetacelloStore classSide methodsFor: 'instance creation' stamp: 'MartinDias 8/23/2011 06:35'!
default

	^default ifNil: [default := self new]! !

!FLMetacelloStore methodsFor: 'exporting' stamp: 'MarianoMartinezPeck 9/24/2011 21:06'!
capturePackagesDuring: aBlock prefixed: aPrefix

	capturedPackageNames := OrderedCollection new.
	prefix := aPrefix.

	capturing := true.
	aBlock value.
	capturing := false.

	self writeFuelPackages.
	self writeLoadScript.! !

!FLMetacelloStore methodsFor: 'initialize-release' stamp: 'MartinDias 8/23/2011 18:10'!
initialize 

	super initialize.
	capturing := false.! !

!FLMetacelloStore methodsFor: 'streaming' stamp: 'MartinDias 8/23/2011 18:05'!
outputDirectory

	^(FileDirectory default directoryNamed: 'FuelMetacello') 
		assureExistence;
		yourself
! !

!FLMetacelloStore methodsFor: 'streaming' stamp: 'MartinDias 8/23/2011 17:58'!
outputFileNamed: aFileName do: aBlock

	^self outputDirectory forceNewFileNamed: aFileName do: aBlock! !

!FLMetacelloStore methodsFor: 'exporting' stamp: 'MartinDias 8/23/2011 13:59'!
packageFileNameFor: packageName
 
	^prefix, '-', packageName, '.fuel'! !

!FLMetacelloStore methodsFor: 'streaming' stamp: 'MartinDias 8/23/2011 18:47'!
packageFileNames

	^capturedPackageNames collect: [:packageName | self packageFileNameFor: packageName]! !

!FLMetacelloStore methodsFor: 'listening' stamp: 'MarianoMartinezPeck 9/24/2011 21:04'!
packagesHasBeenLoaded: packageNames

	capturing ifTrue: [ capturedPackageNames addAll: packageNames]! !

!FLMetacelloStore methodsFor: 'exporting' stamp: 'MartinDias 8/23/2011 17:39'!
scriptFileName
	
	^prefix, '-load-script.txt'! !

!FLMetacelloStore methodsFor: 'exporting' stamp: 'MartinDias 8/23/2011 18:48'!
scriptString

	^ self packageFileNames asArray printString, '
	do: [ :packageFileName | 
		(FileDirectory default directoryNamed: ''FuelMetacello'') 
			oldFileNamed: packageFileName 
			do: [ :aStream | FLPackageLoader new loadFrom: aStream binary ]]
	displayingProgress: [ :packageFileName | ''Fuel importing '', packageFileName ]'! !

!FLMetacelloStore methodsFor: 'exporting' stamp: 'MarianoMartinezPeck 9/24/2011 21:06'!
writeFuelPackages
	capturedPackageNames
		do: [:packageName |
			self outputFileNamed: (self packageFileNameFor: packageName) 
				do: [ :aStream | FLPackageStore new storeOn: aStream binary packageNamed: packageName ] ]
		displayingProgress: [:packageName | 'Capturing ', (self packageFileNameFor: packageName)]
	! !

!FLMetacelloStore methodsFor: 'exporting' stamp: 'MartinDias 8/23/2011 18:05'!
writeLoadScript

	self 
		outputFileNamed: self scriptFileName 
		do: [ :scriptStream | scriptStream nextPutAll: self scriptString ].! !

Object subclass: #FLMethodExtension
	instanceVariableNames: 'method category'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPackageLoader'!

!FLMethodExtension classSide methodsFor: 'instance creation' stamp: 'MartinDias 3/26/2011 04:41'!
with: aCompiledMethod

	^ self basicNew
		initialize;
		method: aCompiledMethod;
		category: aCompiledMethod category;
		yourself ! !

!FLMethodExtension methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 04:40'!
category
	^ category! !

!FLMethodExtension methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 04:40'!
category: anObject
	category := anObject! !

!FLMethodExtension methodsFor: 'loading' stamp: 'MartinDias 9/22/2011 15:39'!
load

	method rebindLiterals.
	method methodClass addSelectorSilently: method selector withMethod: method.
	method methodClass organization classify: method selector under: category.
	! !

!FLMethodExtension methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 04:40'!
method
	^ method! !

!FLMethodExtension methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 04:40'!
method: anObject
	method := anObject! !

Object subclass: #FLPackage
	instanceVariableNames: 'extensionMethods classes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPackageLoader'!

!FLPackage classSide methodsFor: 'private' stamp: 'MartinDias 8/22/2011 02:53'!
classesLocalTo: packageInfo
	^ packageInfo classes! !

!FLPackage classSide methodsFor: 'private' stamp: 'MartinDias 8/21/2011 19:21'!
extensionMethodsTo: packageInfo
	^ packageInfo extensionMethods 
		collect: [ :reference | FLMethodExtension with: reference compiledMethod ]! !

!FLPackage classSide methodsFor: 'instance creation' stamp: 'MartinDias 8/21/2011 19:20'!
named: packageName

	| packageInfo |
	packageInfo := PackageInfo named: packageName.
	^ self basicNew
		initialize;
		classes: (self classesLocalTo: packageInfo);
		extensionMethods: (self extensionMethodsTo: packageInfo);
		yourself! !

!FLPackage methodsFor: 'accessing' stamp: 'MartinDias 8/21/2011 19:10'!
classes
	^ classes! !

!FLPackage methodsFor: 'accessing' stamp: 'MartinDias 8/21/2011 19:10'!
classes: anObject
	classes := anObject! !

!FLPackage methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 03:35'!
extensionMethods
	^ extensionMethods! !

!FLPackage methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 03:35'!
extensionMethods: anObject
	extensionMethods := anObject! !

!FLPackage methodsFor: 'loading' stamp: 'MarianoMartinezPeck 9/23/2011 14:41'!
initializeClass: aClass 
	(aClass class includesSelector: #initialize)
		ifTrue: [aClass theNonMetaClass initialize]! !

!FLPackage methodsFor: 'loading' stamp: 'MartinDias 9/22/2011 13:42'!
initializeClasses
	self classes
		do: [ :aClass | 
			(aClass class includesSelector: #initialize)
				ifTrue: [ aClass theNonMetaClass initialize ] ]! !

!FLPackage methodsFor: 'loading' stamp: 'MarianoMartinezPeck 9/23/2011 15:00'!
load
	self classes
		do: [:aClass | 
			self registerClassInSystemDictionary: aClass.
			self registerClassInSystemOrganization: aClass.
	].
	Smalltalk globals flushClassNameCache.
	self classes do: [:aClass |  self rebindClass: aClass.].
	self classes do: [:aClass | 
		self initializeClass: aClass.
		self notifyClassAddition: aClass.].
	self loadExtensionMethods.
	Behavior flushObsoleteSubclasses! !

!FLPackage methodsFor: 'loading' stamp: 'MartinDias 3/26/2011 04:44'!
loadExtensionMethods
	
	extensionMethods do: [ :extensionMethod | extensionMethod load ]! !

!FLPackage methodsFor: 'loading' stamp: 'MarianoMartinezPeck 9/23/2011 15:03'!
notifyClassAddition: aClass
	SystemChangeNotifier uniqueInstance classAdded: aClass inCategory: aClass category asSymbol.
	! !

!FLPackage methodsFor: 'loading' stamp: 'MarianoMartinezPeck 9/23/2011 14:42'!
rebindClass: aClass 
	aClass rebindMethodDictionary.
	aClass class rebindMethodDictionary! !

!FLPackage methodsFor: 'loading' stamp: 'MartinDias 9/22/2011 15:45'!
rebindClasses
	
	self classes do: [:aClass | 
		aClass rebindMethodDictionary.
		aClass class rebindMethodDictionary. ]! !

!FLPackage methodsFor: 'loading' stamp: 'MarianoMartinezPeck 9/23/2011 14:42'!
registerClassInSystemDictionary: aClass
	Smalltalk globals at: aClass name put: aClass! !

!FLPackage methodsFor: 'loading' stamp: 'MarianoMartinezPeck 9/23/2011 14:42'!
registerClassInSystemOrganization: aClass
	Smalltalk globals organization classify: aClass name under: aClass basicCategory! !

!FLPackage methodsFor: 'loading' stamp: 'MartinDias 8/22/2011 08:07'!
registerInSystemDictionary

	self classes do: [:aClass | Smalltalk globals at: aClass name put: aClass ].
	Smalltalk globals flushClassNameCache.
	! !

!FLPackage methodsFor: 'loading' stamp: 'MartinDias 8/22/2011 08:32'!
registerInSystemOrganization

	self classes do: [:aClass | 
		Smalltalk globals organization classify: aClass name under: aClass basicCategory]! !

Object subclass: #FLPackageLoader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPackageLoader'!

!FLPackageLoader methodsFor: 'loading' stamp: 'MartinDias 9/13/2011 22:01'!
loadFrom: aStream 

	| package |
	package := (FLMaterializer newDefault materializeFrom: aStream) root.
	package load.
	^ package.! !

Object subclass: #FLPackageStore
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPackageLoader'!

!FLPackageStore methodsFor: 'storing' stamp: 'MartinDias 9/13/2011 22:02'!
storeOn: aStream packageNamed: name 

	| package |
	package := FLPackage named: name.
	(FLSerializer newDefault)
		analyzer: (FLPackageAnalyzer newDefault 
			considerInternalAll: package classes; 
			considerGlobalWhenPossible;
			yourself);
		serialize: package on: aStream.! !

!Object methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 9/22/2011 02:12'!
rebindOn: aCompiledMethod at: index
	"Nothing to do."! !

!ClassDescription methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 02:42'!
foreignCategories
	
	^self organization categories select: [ :aCategory | aCategory beginsWith: '*' ]! !

!ClassDescription methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 01:50'!
removeCategorySilently: aCategory

	(organization listAtCategoryNamed: aCategory) do: [ :aSelector | self removeSelectorSilently: aSelector ].
	organization removeCategorySilently: aCategory.
! !

!ClassDescription methodsFor: '*FuelPackageLoader' stamp: 'MartinDias 8/22/2011 03:05'!
removeForeignProtocol

	self foreignCategories do: [ :aCategory |
		self removeCategorySilently: aCategory ]! !
